@@ -44,12 +44,12 @@ execute()
 
 	# Run the command
 	if [ "$profile" -ge "0" ] && [ "$profile" -le "1" ]; then 
-		ffmpeg -f dv -i pipe: -pass $pass -passlogfile "$file" -f 3gp \
+		ffmpeg -threads $threads -f dv -i pipe: -pass $pass -passlogfile "$file" -f 3gp \
 		-vcodec mpeg4 $hq -b 30$kilo -minrate 0 -maxrate 32$kilo -bufsize 20$bufsize \
 		-r 5 $progressive -s qcif -aspect $aspect -g 10 \
 		-acodec $aac -ab 12$audio_kilo -ar 8000 -ac 1 -y "$file".3gp
 	else
-		ffmpeg -f dv -i pipe: -pass $pass -passlogfile "$file" -f 3gp \
+		ffmpeg -threads $threads -f dv -i pipe: -pass $pass -passlogfile "$file" -f 3gp \
 		-vcodec mpeg4 $hq -b 90$kilo -minrate 0 -maxrate 100$kilo -bufsize 40$bufsize \
 		-r 12 $progressive -s qcif -aspect $aspect -g 24 \
 		-acodec $aac -ab 20$audio_kilo -ar 12000 -ac 1 -y "$file".3gp
