@@ -1,4 +1,6 @@
 #!/bin/sh
+# A Kino export script that outputs to DVD-Video with ffmpeg and mplex,
+# optionally processed with dvdauthor
 
 usage()
 {
@@ -18,12 +20,10 @@ usage()
 	echo Flags: double-pass file-producer
 
 	# Profiles
-	echo "Profile: Standard VOB"
-	echo "Profile: Widescreen VOB"
+	echo "Profile: VOB"
 	which dvdauthor > /dev/null
 	if [ $? -eq 0 ]; then
-		echo "Profile: Standard DVD-Video directory (All only)"
-		echo "Profile: Widescreen DVD-Video directory (All only)"
+		echo "Profile: DVD-Video directory (All only)"
 	fi
 }
 
@@ -36,8 +36,9 @@ execute()
 	file="$4"
 	smil_file="$5"
 	pass="$6"
+	aspect="$7"
 
-	source "`dirname $0`/ffmpeg_utils.sh"
+	. "`dirname $0`/ffmpeg_utils.sh"
 
 	# generate filename if missing
 	[ "x$file" = "x" ] && file="kino_export_"`date +%Y-%m-%d_%H.%M.%S`
@@ -46,7 +47,7 @@ execute()
 	# Determine info arguments
 	frame_rate=`[ "$normalisation" = "pal" ] && echo 25 || echo 29.97`
 	audio_format="ac3"
-	audio_bitrate="192"
+	audio_bitrate="192$audio_kilo"
 
 	# Build path to extract_chapters Perl script
 	get_chapters="`dirname $0`/extract_chapters"
@@ -63,12 +64,12 @@ execute()
 				ffmpeg -v 0 -f dv -i pipe: -f rawvideo -pix_fmt yuv420p pipe: \
 				-vn -f $audio_format -ac 2 -ab $audio_bitrate -ar 48000 -y "$AC3" | \
 				ffmpeg -v 0 -f rawvideo -pix_fmt yuv420p -s "$normalisation" -r "$normalisation" -i pipe: \
-				-an -target dvd -f mpeg1video -maxrate 8000 -ildct -ilme -aspect 4:3 -pass 1 -passlogfile "$LOG" -y "$M2V"
+				-an -target dvd -f mpeg2video -maxrate 8000$kilo -ildct -ilme -aspect $aspect -pass 1 -passlogfile "$LOG" -y "$M2V"
 			else
 				if [ $pass -eq "2" ]; then
 					ffmpeg -v 0 -f dv -i pipe: -an -f rawvideo -pix_fmt yuv420p pipe: | \
 					ffmpeg -v 0 -f rawvideo -pix_fmt yuv420p -s "$normalisation" -r "$normalisation" -i pipe: \
-					-an -target dvd -f mpeg1video -maxrate 8000 -ildct -ilme -aspect 4:3 -pass 2 -passlogfile "$LOG" -y "$M2V"
+					-an -target dvd -f mpeg2video -maxrate 8000$kilo $interlace -aspect $aspect -pass 2 -passlogfile "$LOG" -y "$M2V"
 					mplex -v 0 -f 8 -o "$file".vob "$M2V" "$AC3"
 					rm -f "$M2V" "$AC3" "$LOG-0.log"
 				fi
@@ -79,57 +80,27 @@ execute()
 				ffmpeg -v 0 -f dv -i pipe: -f rawvideo -pix_fmt yuv420p pipe: \
 				-vn -f $audio_format -ac 2 -ab $audio_bitrate -ar 48000 -y "$AC3" | \
 				ffmpeg -v 0 -f rawvideo -pix_fmt yuv420p -s "$normalisation" -r "$normalisation" -i pipe: \
-				-an -target dvd -f mpeg1video -maxrate 8000 -ildct -ilme -aspect 16:9 -pass 1 -passlogfile "$LOG" -y "$M2V"
-			else
-				if [ $pass -eq "2" ]; then
-					ffmpeg -v 0 -f dv -i pipe: -an -f rawvideo -pix_fmt yuv420p pipe: | \
-					ffmpeg -v 0 -f rawvideo -pix_fmt yuv420p -s "$normalisation" -r "$normalisation" -i pipe: \
-					-an -target dvd -f mpeg1video -maxrate 8000 -ildct -ilme -aspect 16:9 -pass 2 -passlogfile "$LOG" -y "$M2V"
-					mplex -v 0 -f 8 -o "$file".vob "$M2V" "$AC3"
-					rm -f "$M2V" "$AC3" "$LOG-0.log"
-				fi
-			fi
-			;;
-		"2" )
-			if [ $pass -eq "1" ]; then
-				ffmpeg -v 0 -f dv -i pipe: -f rawvideo -pix_fmt yuv420p pipe: \
-				-vn -f $audio_format -ac 2 -ab $audio_bitrate -ar 48000 -y "$AC3" | \
-				ffmpeg -v 0 -f rawvideo -pix_fmt yuv420p -s "$normalisation" -r "$normalisation" -i pipe: \
-				-an -target dvd -f mpeg1video -maxrate 8000 -ildct -ilme -aspect 4:3 -pass 1 -passlogfile "$LOG" -y "$M2V"
+				-an -target dvd -f mpeg2video -maxrate 8000$kilo $interlace -aspect $aspect -pass 1 -passlogfile "$LOG" -y "$M2V"
 			else
 				if [ $pass -eq "2" ]; then
 					VOB=`mktemp -u $file.XXXXXX`
 					mkfifo "$VOB"
 					rm -rf "$file"
+
 					ffmpeg -v 0 -f dv -i pipe: -an -f rawvideo -pix_fmt yuv420p pipe: | \
 					ffmpeg -v 0 -f rawvideo -pix_fmt yuv420p -s "$normalisation" -r "$normalisation" -i pipe: \
-					-an -target dvd -f mpeg1video -maxrate 8000 -ildct -ilme -aspect 4:3 -pass 2 -passlogfile "$LOG" -y "$M2V"
+					-an -target dvd -f mpeg2video -maxrate 8000$kilo $interlace -aspect $aspect -pass 2 -passlogfile "$LOG" -y "$M2V"
 					mplex -v 0 -f 8 -o "$VOB" "$M2V" "$AC3" &
-					dvdauthor -o "$file" --chapters="`$get_chapters -f $frame_rate -x $smil_file`" "$VOB"
+
+					XML=`mktemp $file-dvdauthor.XXXXXX`
+					CHAPTERS="`$get_chapters -f $frame_rate -x $smil_file`"
+					echo "<?xml version=\"1.0\"?>" > "$XML"
+					echo "<dvdauthor><vmgm><menus><video/><audio/><subpicture lang=\"en\"/></menus></vmgm><titleset><titles><pgc pause=\"0\">" >> "$XML"
+					echo "<vob file=\"$VOB\" chapters=\"$CHAPTERS\" pause=\"0\"/>" >> "$XML"
+					echo "</pgc></titles></titleset></dvdauthor>" >> "$XML"
+					dvdauthor -o "$file" -x "$XML"
 					dvdauthor -o "$file" -T
-					rm -f "$VOB"
-					rm -f "$M2V" "$AC3" "$LOG-0.log"
-				fi
-			fi
-			;;
-		"3" )
-			if [ $pass -eq "1" ]; then
-				ffmpeg -v 0 -f dv -i pipe: -f rawvideo -pix_fmt yuv420p pipe: \
-				-vn -f $audio_format -ac 2 -ab $audio_bitrate -ar 48000 -y "$AC3" | \
-				ffmpeg -v 0 -f rawvideo -pix_fmt yuv420p -s "$normalisation" -r "$normalisation" -i pipe: \
-				-an -target dvd -f mpeg1video -maxrate 8000 -ildct -ilme -aspect 16:9 -pass 1 -passlogfile "$LOG" -y "$M2V"
-			else
-				if [ $pass -eq "2" ]; then
-					VOB=`mktemp -u $file.XXXXXX`
-					mkfifo "$VOB"
-					rm -rf "$file"
-					ffmpeg -v 0 -f dv -i pipe: -an -f rawvideo -pix_fmt yuv420p pipe: | \
-					ffmpeg -v 0 -f rawvideo -pix_fmt yuv420p -s "$normalisation" -r "$normalisation" -i pipe: \
-					-an -target dvd -f mpeg1video -maxrate 8000 -ildct -ilme -aspect 16:9 -pass 2 -passlogfile "$LOG" -y "$M2V"
-					mplex -v 0 -f 8 -o "$VOB" "$M2V" "$AC3" &
-					dvdauthor -o "$file" --chapters="`$get_chapters -f $frame_rate -x $smil_file`" "$VOB"
-					dvdauthor -o "$file" -T
-					rm -f "$VOB"
+					rm -f "$VOB" "$XML"
 					rm -f "$M2V" "$AC3" "$LOG-0.log"
 				fi
 			fi
