@@ -63,6 +63,8 @@ stty -echo
 read -p "  Password: " password
 echo
 stty echo
+echo
+read -p "Do you want higher quality (slower and bigger to convert & upload)? [y/N] " quality
 
 title=$(getXmlAttributeValue 'title' "$project_file")
 [ -z "$title" ] && title="Untitled"
@@ -103,10 +105,44 @@ else
 		med_res_y="240"
 	fi
 fi
-kino2raw "$project_file" |
-	ffmpeg2theora -f dv -p preview --optimize -o "$ogg_file" --artist "$author" --title "$title" --location "$abstract" -
-	# Example of custom output
-	# ffmpeg2theora -f dv --cropleft 8 --cropright 8 --optimize -x $full_res_x -y $full_res_y -K 200 -v 5 -S 2 -a 2 -o "$ogg_file" --artist "$author" --title "$title" --location "$abstract" -
+
+# Try ffmpeg2theora first
+which ffmpeg2theora 2> /dev/null > /dev/null
+if [ $? -eq 0 ]; then
+	if [ -z "$quality" ] || [ "$quality" = "n" ] || [ "$quality" = "N" ] || [ "$quality" = "no" ] || [ "$quality" = "No" ] || [ "$quality" = "NO" ]; then
+		kino2raw "$project_file" |
+		ffmpeg2theora -f dv -p preview --optimize -o "$ogg_file" --artist "$author" --title "$title" --location "$abstract" -
+	else
+		kino2raw "$project_file" |
+		ffmpeg2theora -f dv --cropleft 8 --cropright 8 --optimize -x $full_res_x -y $full_res_y \
+			-K 200 -v 5 -S 2 -a 2 -o "$ogg_file" --artist "$author" --title "$title" --location "$abstract" -
+	fi
+else
+	# Try gstreamer next
+	. "$(dirname $0)/../exports/gstreamer_utils.sh"
+
+	# Run the transcode
+	if [ -z "$quality" ] || [ "$quality" = "n" ] || [ "$quality" = "N" ] || [ "$quality" = "no" ] || [ "$quality" = "No" ] || [ "$quality" = "NO" ]; then
+		# compute other framerates
+		halffps=$( [ "$normalisation" = "pal" ] && echo "25/2" || echo "30000/2002" )
+	
+		kino2raw "$project_file" |
+		$gstlaunch filesrc use-mmap=false location=/dev/stdin ! dvdemux name=demux ! queue ! \
+			video/x-dv,systemstream=\(boolean\)false ! $dvdec ! videorate ! \
+			video/x-raw-yuv,framerate=$halffps ! ffmpegcolorspace ! videocrop left=8 right=8 ! \
+			videoscale method=0 ! video/x-raw-yuv,width=$med_res_x,height=$med_res_y,pixel-aspect-ratio=\(fraction\)1/1 ! \
+			theoraenc bitrate=500 keyframe-freq=200 ! oggmux name=mux ! filesink location="$ogg_file" demux. ! \
+			audio/x-raw-int ! queue ! audioresample ! audio/x-raw-int,rate=32000 ! audioconvert ! vorbisenc max-bitrate=64000 ! mux.
+	else
+		kino2raw "$project_file" |
+		$gstlaunch filesrc use-mmap=false location=/dev/stdin ! dvdemux name=demux ! queue ! \
+			video/x-dv,systemstream=\(boolean\)false ! $dvdec ! \
+			ffmpegcolorspace ! videocrop left=8 right=8 ! $deinterlace \
+			videoscale method=1 ! video/x-raw-yuv,width=$full_res_x,height=$full_res_y,pixel-aspect-ratio=\(fraction\)1/1 ! \
+			theoraenc quality=44 ! oggmux name=mux ! filesink location="$ogg_file" demux. ! \
+			audio/x-raw-int ! queue ! audioconvert ! vorbisenc quality=0.3 ! mux.
+	fi
+fi
 
 echo
 echo Uploading to blip.tv
