@@ -1,6 +1,7 @@
 /*
 * filehandler.cc -- saving DV data into different file formats
 * Copyright (C) 2000 Arne Schirmacher <arne@schirmacher.de>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -422,10 +423,12 @@ int RawHandler::GetFrame( Frame &frame, int frameNum )
 
 /***************************************************************************/
 
+#define AVI_AUDIO_BUFFER_SECS (2)
 
 AVIHandler::AVIHandler( int format ) : avi( NULL ), aviFormat( format ), isOpenDML( false ),
 		fccHandler( make_fourcc( "dvsd" ) ), channels( 2 ), isFullyInitialized( false ),
-		audioBuffer( NULL )
+		audioBuffer( NULL ), isInterleave1to1( true ), audioOffset( 0 ),
+		audioBufferSize( 0 )
 {
 	extension = ".avi";
 	for ( int c = 0; c < 4; c++ )
@@ -597,9 +600,14 @@ bool AVIHandler::Open( const char *s )
 			return false;
 		avi->ReadIndex();
 		if ( avi->verifyStream( make_fourcc( "auds" ) ) )
+		{
 			aviFormat = AVI_DV2_FORMAT;
+			isInterleave1to1 = avi->isInterleave1to1();
+		}
 		else
+		{
 			aviFormat = AVI_DV1_FORMAT;
+		}
 		isOpenDML = avi->isOpenDML();
 		filename = s;
 		return true;
@@ -618,14 +626,15 @@ int AVIHandler::GetFrame( Frame &frame, int frameNum )
 		if ( aviFormat == AVI_DV2_FORMAT )
 		{
 			WAVEFORMATEX wav;
+			FOURCC fccWav = make_fourcc( "01wb" );
 
 			if ( ! isFullyInitialized && 
 				 avi->getStreamFormat( ( void* ) &wav, make_fourcc( "auds" ) ) )
 			{
 				if ( channels > 0 && channels < 5 )
 				{
-					// Allocate interleaved audio buffer
-					audioBuffer = new int16_t[ 2 * DV_AUDIO_MAX_SAMPLES * channels ];
+					// Allocate interleaved audio buffer to accomodate large chunks
+					audioBuffer = new int16_t[ wav.nSamplesPerSec * channels * AVI_AUDIO_BUFFER_SECS ];
 
 					// Allocate non-interleaved audio buffers
 					for ( int c = 0; c < channels; c++ )
@@ -642,32 +651,125 @@ int AVIHandler::GetFrame( Frame &frame, int frameNum )
 			}
 
 			// Get the frame from AVI
-			int n = avi->getFrame( audioBuffer, frameNum, make_fourcc( "01wb" ) );
-			if ( n > 0 )
+			if ( isInterleave1to1 )
 			{
-				// Temporary pointer to audio scratch buffer
-				int16_t * s = audioBuffer;
-
-				// Determine samples in this frame
-				audioInfo.samples = n / audioInfo.channels / sizeof( int16_t );
-				
-				// Convert interleaved audio into non-interleaved
-				for ( int n = 0; n < audioInfo.samples; ++n )
-					for ( int i = 0; i < audioInfo.channels; i++ )
-						audioChannels[ i ][ n ] = *s++;
-
-				// Write interleaved audio into frame
-				frame.EncodeAudio( audioInfo, audioChannels );
+				int n = avi->getFrame( audioBuffer, frameNum, fccWav );
+				if ( n > 0 )
+				{
+					// Temporary pointer to audio scratch buffer
+					int16_t * s = audioBuffer;
+	
+					// Determine samples in this frame
+					audioInfo.samples = n / audioInfo.channels / sizeof( int16_t );
+					if ( audioInfo.samples > DV_AUDIO_MAX_SAMPLES )
+						audioInfo.samples = DV_AUDIO_MAX_SAMPLES;
+					
+					// Convert interleaved audio into non-interleaved
+					for ( int n = 0; n < audioInfo.samples; ++n )
+						for ( int i = 0; i < audioInfo.channels; i++ )
+							audioChannels[ i ][ n ] = *s++;
+				}
 			}
-		}
+			else
+			{
+				int16_t* s;
+				int availSamples = 0;
 
-		// Parse important metadata in DV bitstream
-		frame.ExtractHeader();
+				// Fetch correct audio chunk and get audio buffer offset
+				if ( ( s = SeekAudioFrame( frame, frameNum, availSamples ) ) )
+				{
+					if ( availSamples < audioInfo.samples )
+					{
+						// Not enough samples in audio buffer
+						int n;
+
+						// Convert interleaved samples remaining in the buffer
+						for ( n = 0; n < availSamples; ++n )
+							for ( int i = 0; i < audioInfo.channels; i++ )
+								audioChannels[ i ][ n ] = *s++;
+
+						// Load the next audio chunk into buffer
+						if ( ( s = SeekAudioFrame( frame, frameNum + 1, availSamples ) ) )
+						{
+							// Convert interleaved audio into non-interleaved
+							for ( --n; n < audioInfo.samples; ++n )
+								for ( int i = 0; i < audioInfo.channels; i++ )
+									audioChannels[ i ][ n ] = *s++;
+						}
+						else if ( availSamples >= 0 )
+						{
+							// Just truncate number of samples in this frame
+							audioInfo.samples = availSamples;
+						}
+					}
+					else
+					{
+						// Convert interleaved audio into non-interleaved
+						for ( int n = 0; n < audioInfo.samples; ++n )
+							for ( int i = 0; i < audioInfo.channels; i++ )
+								audioChannels[ i ][ n ] = *s++;
+					}	
+				}
+				// <0 signals an error in SeekAudioFrame
+				if ( availSamples < 0 )
+					return result;
+			}
+			// Write interleaved audio into frame
+			frame.ExtractHeader();
+			frame.EncodeAudio( audioInfo, audioChannels );
+		}
+		else
+		{
+			// Parse important metadata in DV bitstream
+			frame.ExtractHeader();
+		}
 	}
 	return result;
 }
 
 
+int16_t* AVIHandler::SeekAudioFrame( Frame& frame, int frameNum, int& availableSize )
+{
+	// Not very optimized - needs to seek to PCM sample offset
+	FOURCC fccWav = make_fourcc( "01wb" );
+	frame.ExtractHeader();
+	off_t offsetBytes = off_t( 0.5 + (float) frameNum / frame.GetFrameRate() * audioInfo.frequency );
+	offsetBytes *= audioInfo.channels * sizeof(int16_t);
+
+	// If cache miss
+	if ( offsetBytes < audioOffset || offsetBytes >= ( audioOffset + audioBufferSize ) )
+	{
+		int i = 0;
+		off_t dummy;
+
+		// Find chunk corresponding to offset
+		for ( audioOffset = 0; avi->GetFrameInfo( dummy, audioBufferSize, i, fccWav ) != -1; ++i )
+		{
+			if ( audioBufferSize <= 0 )
+			{
+				availableSize = -1;
+				return 0;
+			}
+			if ( audioOffset + audioBufferSize > offsetBytes )
+				break;
+			audioOffset += audioBufferSize;
+		}
+		// Read chunk if buffer is large enough to hold it
+		if ( audioBufferSize > ( off_t( audioInfo.frequency ) * audioInfo.channels * sizeof(int16_t) * AVI_AUDIO_BUFFER_SECS ) ||
+		     !avi->getFrame( audioBuffer, i, fccWav ) )
+		{
+			return 0;
+		}
+	}
+	// Return how many samples in buffer are available after seeking
+	availableSize = ( audioBufferSize - int( offsetBytes - audioOffset ) ) >> 2;
+	// Set number of samples to use in this frame
+	audioInfo.samples = frame.CalculateNumberSamples( audioInfo.frequency, frameNum );
+	// Return offset into audio buffer
+	return audioBuffer + ( ( offsetBytes - audioOffset ) >> 1 );
+}
+
+
 void AVIHandler::SetOpenDML( bool flag )
 {
 	isOpenDML = flag;
@@ -702,7 +804,7 @@ void QtHandler::Init()
 		Close();
 
 	fd = NULL;
-	samplingRate = 0;
+	samplingRate = 48000;
 	samplesPerBuffer = 0;
 	channels = 2;
 	audioBuffer = NULL;
@@ -771,6 +873,7 @@ int QtHandler::Write( const Frame& frame )
 		if ( frame.GetAudioInfo( audio ) )
 		{
 			channels = 2;
+			samplingRate = audio.frequency;
 			quicktime_set_audio( fd, channels, audio.frequency, 16, QUICKTIME_TWOS );
 		}
 		else
@@ -793,10 +896,20 @@ int QtHandler::Write( const Frame& frame )
 		{
 			long bytesRead = frame.ExtractAudio( audioBuffer );
 
-			DeinterlaceStereo16( audioBuffer, bytesRead,
-			                     audioChannelBuffer[ 0 ],
-			                     audioChannelBuffer[ 1 ] );
-
+			if ( bytesRead )
+			{
+				DeinterlaceStereo16( audioBuffer, bytesRead,
+									audioChannelBuffer[ 0 ],
+									audioChannelBuffer[ 1 ] );
+			}
+			else
+			{
+				// XXX: desired sample rate is unkown if not initialized with frame
+				// carrying audio.
+				audio.samples = int( (float)samplingRate / frame.GetFrameRate() );
+				memset( audioChannelBuffer[0], 0, audio.samples * sizeof(int16_t) );
+				memset( audioChannelBuffer[1], 0, audio.samples * sizeof(int16_t) );
+			}
 			quicktime_encode_audio( fd, audioChannelBuffer, NULL, audio.samples );
 		}
 	}
@@ -903,9 +1016,13 @@ int QtHandler::GetFrame( Frame &frame, int frameNum )
 			quicktime_set_audio_position( fd, ( int64_t )( frameNum * samples ), 0 );
 			quicktime_decode_audio( fd, audioChannelBuffer[ i ], NULL, (long) samples, i );
 		}
+		frame.ExtractHeader();
 		frame.EncodeAudio( info, audioChannelBuffer );
 	}
-	frame.ExtractHeader();
+	else
+	{
+		frame.ExtractHeader();
+	}
 	return 0;
 }
 #endif
