@@ -97,7 +97,6 @@ Frame::Frame() : bytesInFrame( 0 )
 	pthread_mutex_lock( &avcodec_mutex );
 	av_register_all();
 	libavcodec = avcodec_alloc_context();
-	libavcodec->thread_count = 2;
 	avcodec_open( libavcodec, avcodec_find_decoder( CODEC_ID_DVVIDEO ) );
 	pthread_mutex_unlock( &avcodec_mutex );
 	data = ( unsigned char* ) av_mallocz( 144000 );
@@ -1332,6 +1331,7 @@ bool Frame::CreateEncoder( bool isPAL, bool isWide )
  					avcodecEncoder->sample_aspect_ratio = isWide ? ( AVRational ){ 40, 33 } : ( AVRational ){ 10, 11 };
 			}
 			avcodecEncoder->thread_count = 2;
+			avcodec_thread_init( avcodecEncoder, avcodecEncoder->thread_count );
 			avcodecEncoder->time_base= isPAL ? ( AVRational ){ 1, 25 } : ( AVRational ){ 1001, 30000 };
 			avcodecEncoder->pix_fmt = isPAL ? PIX_FMT_YUV420P : PIX_FMT_YUV411P;
 			avcodecEncoder->flags |= CODEC_FLAG_INTERLACED_DCT;
