@@ -1,6 +1,7 @@
 /*
 * image_create.cc -- RGB24 image create
 * Copyright (C) 2002 Charles Yates <charles.yates@pandora.be>
+* Copyright (C) 2002-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -21,6 +22,8 @@
 #include <config.h>
 #endif
 
+#include <iostream>
+
 #include <glade/glade.h>
 #include <string.h>
 #include <stdio.h>
@@ -33,6 +36,8 @@
 #include "message.h"
 #include "kino_extra.h"
 
+#define GAP_MAX (9999)
+
 extern "C"
 {
 #include "support.h"
@@ -66,11 +71,6 @@ public:
 		frames = 25;
 	}
 
-	static void Repaint( void )
-	{
-		GetSelectedFramesForFX().Repaint();
-	}
-
 	char *GetDescription( ) const
 	{
 		return _( "Fixed Colour" );
@@ -342,14 +342,9 @@ public:
 		gtk_widget_destroy( window );
 	}
 
-	static void Repaint( void )
-	{
-		GetSelectedFramesForFX().Repaint();
-	}
-
 	char *GetDescription( ) const
 	{
-		return _( "Gradiant" );
+		return _( "Gradient" );
 	}
 
 	void LeftMiddle( uint8_t *pixels, int width, int height, double position )
@@ -507,75 +502,117 @@ protected:
 	static char file[ PATH_MAX + NAME_MAX ];
 	GtkWidget *file_entry;
 	int count;
+	GdkPixbuf* still;
+	bool hasFilenameChanged;
+	int count_offset;
 
 public:
-	ImageCreateFromFile( ) : file_entry( 0 )
+	ImageCreateFromFile( )
+		: file_entry( 0 )
+		, still( 0 ) 
+		, hasFilenameChanged( false )
+		, count_offset( 0 )
 	{ }
 
 	char *GetDescription( ) const
 	{
-		return _( "Create From File" );
+		return _( "From File" );
 	}
 
 	void CreateFrame( uint8_t *pixels, int width, int height, double position, double frame_delta )
 	{
-		GError* gerror = NULL;
-		GdkPixbuf* still = NULL;
-
 		if ( strcmp( file, "" ) == 0 )
 			return;
 		
+		GError* gerror = NULL;
+		SelectedFrames& fx = GetSelectedFramesForFX();
+
 		// Determine if this is an image sequence
 		if ( strchr( file, '%' ) != NULL )
 		{
 			char full[ PATH_MAX + NAME_MAX ];
 			int gap = 0;
 		
-			// Permit gaps in the file number sequence up to 100 difference
-			while ( gap < 100 )
+			if ( fx.IsRepainting() || fx.IsPreviewing() )
+			{
+				// compute the relative count
+				count = int( position * GetNumberOfFrames() + 0.5 );
+				
+				// determine offset to first file
+				if ( hasFilenameChanged )
+				for ( count_offset = 0; count_offset < GAP_MAX; ++count_offset )
+				{
+					struct stat buf;
+					snprintf( full, PATH_MAX + NAME_MAX, file, count + count_offset );
+					if ( stat( full, &buf ) == 0 )
+						break;
+				}
+				count += count_offset;
+			}
+			// Permit gaps in the file number sequence
+			while ( gap < GAP_MAX )
 			{
 				struct stat buf;
-					
 				snprintf( full, PATH_MAX + NAME_MAX, file, count++ );
 				if ( stat( full, &buf ) == 0 )
 					break;
 				else
 					gap ++;
 			}
-			
 			// If successfully obtained a new file
-			if ( gap < 100 )
-				still = gdk_pixbuf_new_from_file( full, &gerror );
+			if ( gap < GAP_MAX )
+			{
+				GdkPixbuf *image = gdk_pixbuf_new_from_file( full, &gerror );
+				if ( still )
+					g_object_unref( still );
+				still = gdk_pixbuf_scale_simple( image, width, height, GDK_INTERP_HYPER );
+				g_object_unref( image );
+			}
 		}
 		// Otherwise, import single image just once
-		else if ( position == 0 )
+		else if ( hasFilenameChanged )
 		{
-			still = gdk_pixbuf_new_from_file( file, &gerror );
+			GdkPixbuf *image = gdk_pixbuf_new_from_file( file, &gerror );
+			if ( still )
+				g_object_unref( still );
+			if ( image )
+			{
+				still = gdk_pixbuf_scale_simple( image, width, height, GDK_INTERP_HYPER );
+				g_object_unref( image );
+			}
 		}
 		// If gdk-pixbuf loader generated an error
 		if ( gerror != NULL )
 		{
-			modal_message( gerror->message );
+			if ( hasFilenameChanged )
+			{
+				if ( fx.IsPreviewing() )
+					gdk_threads_enter();
+				modal_message( gerror->message );
+				if ( fx.IsPreviewing() )
+					gdk_threads_leave();
+			}
 			g_error_free( gerror );
+			if ( still )
+				g_object_unref( still );
+			still = 0;
 		}
 		// Otherwise, if there is a pixbuf
-		else if ( still != NULL )
+		else if ( still )
 		{
-			GdkPixbuf *image = gdk_pixbuf_scale_simple( still, width, height, GDK_INTERP_HYPER );
-			if ( gdk_pixbuf_get_has_alpha( image ) ) {
+			if ( gdk_pixbuf_get_has_alpha( still ) ) {
 			// This is a hack! The correct calls would be:
 			// gdk_pixbuf_new/gdk_pixbuf_copy_area
-				uint8_t *jp = gdk_pixbuf_get_pixels( image ), *jz = pixels;
+				uint8_t *jp = gdk_pixbuf_get_pixels( still ), *jz = pixels;
 				for ( int i = 0; i < width * height; i++ ) {
 					memcpy( jz, jp, 3 );
 					jp += 4; jz += 3;
 				}
 			}
 			else
-				memcpy( pixels, gdk_pixbuf_get_pixels( image ), width * height * 3 );
-			g_object_unref( still );
-			g_object_unref( image );
+				memcpy( pixels, gdk_pixbuf_get_pixels( still ), width * height * 3 );
 		}
+		hasFilenameChanged = false;
 	}
 
 	static void
@@ -601,7 +638,7 @@ public:
 		GtkWidget *hbox = gtk_hbox_new( FALSE, 6 );
 		file_entry = gtk_entry_new();
 		gtk_entry_set_text( GTK_ENTRY( file_entry ), file );
-		g_signal_connect( G_OBJECT( file_entry ), "changed", G_CALLBACK( Repaint ), 0 );
+		g_signal_connect( G_OBJECT( file_entry ), "activate", G_CALLBACK( Repaint ), 0 );
 		gtk_box_pack_start( GTK_BOX( hbox ), file_entry, TRUE, TRUE, 0 );
 		GtkWidget *button = gtk_button_new();
 		GtkWidget *image = gtk_image_new_from_stock( GTK_STOCK_OPEN, GTK_ICON_SIZE_BUTTON );
@@ -648,7 +685,14 @@ public:
 			InterpretFrameCollector( );
 			if ( file_entry )
 			{
-				strcpy( file, gtk_entry_get_text( GTK_ENTRY( file_entry ) ) );
+				int n = PATH_MAX + NAME_MAX - 1;
+				char newFileName[ n + 1 ];
+				newFileName[ n ] = '\0';
+				strncpy( newFileName, gtk_entry_get_text( GTK_ENTRY( file_entry ) ), n );
+				SelectedFrames& fx = GetSelectedFramesForFX();
+				if ( strcmp( newFileName, file ) || !( fx.IsRepainting() || fx.IsPreviewing() ) )
+					hasFilenameChanged = true;
+				strcpy( file, newFileName );
 				count = 0;
 			}
 		}
@@ -681,13 +725,12 @@ on_optionmenu_selected ( GtkMenuItem *menu_item, gpointer user_data )
 
 GDKImageCreateRepository::GDKImageCreateRepository() : selected_creator( NULL ), menu( NULL ), container( NULL )
 {
-	printf( ">> image creator repository created\n" );
 	// Register an instance of each object
-	Register( new ImageCreateColour() );
-	Register( new ImageCreateNoise() );
 	Register( new ImageCreateColourRange() );
-	Register( new ImageCreateGradiate() );
+	Register( new ImageCreateColour() );
 	Register( new ImageCreateFromFile() );
+	Register( new ImageCreateGradiate() );
+	Register( new ImageCreateNoise() );
 }
 
 /** Destructor for the image repository - clears all the registered creators
@@ -695,7 +738,6 @@ GDKImageCreateRepository::GDKImageCreateRepository() : selected_creator( NULL ),
 
 GDKImageCreateRepository::~GDKImageCreateRepository()
 {
-	printf( ">> image creator repository destroyed\n" );
 	// Remove the creators in the repository
 	for ( unsigned int index = 0; index < creators.size(); index ++ )
 		delete creators[ index ];
@@ -706,7 +748,7 @@ GDKImageCreateRepository::~GDKImageCreateRepository()
 
 void GDKImageCreateRepository::Register( GDKImageCreate *creator )
 {
-	printf( ">>> Image Create: %s\n", creator->GetDescription( ) );
+	std::cerr <<  ">>> Image Create: " << creator->GetDescription( ) << std::endl;
 	creators.push_back( creator );
 }
 
@@ -757,11 +799,11 @@ void GDKImageCreateRepository::SelectionChange( )
 	// Get the new selection
 	selected_creator = Get();
 
-	// Inform the main page of the change
-	if ( common != NULL && common->getPageMagick( ) != NULL )
-		common->getPageMagick( ) ->RefreshStatus( );
-
 	// Attach the new creators widgets
 	if ( selected_creator != NULL )
 		selected_creator->AttachWidgets( container );
+
+	// Inform the main page of the change
+	if ( common != NULL && common->getPageMagick( ) != NULL )
+		common->getPageMagick( ) ->RefreshStatus( );
 }
