@@ -66,6 +66,7 @@ using std::endl;
 #include "ieee1394io.h"
 #include "error.h"
 #include "stringutils.h"
+#include "rwpipe.h"
 
 
 extern "C"
@@ -338,7 +339,7 @@ bool KinoCommon::newFile( bool prompt, bool isExiting )
 			for ( it = list.begin(); it != list.end(); it ++ )
 				buffer += *it + "\n";
 
-			switch ( modal_confirm_files( GTK_STOCK_DELETE, GTK_STOCK_CLOSE, buffer.c_str(), 
+			switch ( modal_confirm_files( GTK_STOCK_DELETE, isExiting? GTK_STOCK_QUIT: GTK_STOCK_NEW, buffer.c_str(), 
 				_( "The following FX rendered files are no longer used.\nDo you want me to delete them now?" ) ) )
 			{
 			case GTK_RESPONSE_ACCEPT:
@@ -581,6 +582,19 @@ std::string KinoCommon::importFile( const char *filename )
 		char *kinoHome = getenv("KINO_HOME");
 		string homeFile;
 		int testHomeFile = 0;
+		RWPipe pipe;
+		std::ostringstream command;
+		char duration[ 13 ];
+
+		// Get the duration as a time value string
+		command << "\"" << DATADIR << "/kino/scripts/import/duration.sh\" ";
+		command << "\"" << filename << "\"" << std::ends;
+		duration[0] = 0;
+		if ( pipe.run( command.str() ) )
+		{
+			while ( pipe.readLine( duration, sizeof(duration) ) == 0 );
+			pipe.stop();
+		}
 
 		if ( kinoHome )
 			homeFile = string( kinoHome ) + string( "/import/media.sh" );
@@ -588,7 +602,8 @@ std::string KinoCommon::importFile( const char *filename )
 			homeFile = string( getenv( "HOME" ) ) + string( "/kino/import/media.sh" );
 
 		gtk_window_set_transient_for( GTK_WINDOW( dialog ), GTK_WINDOW( getWidgetWindow( getWidget() ) ) );
-		gtk_progress_bar_pulse( progressBar );
+		gtk_progress_bar_set_fraction( progressBar, 0 );
+		gtk_progress_bar_set_text( progressBar, "" );
 		gtk_widget_show_all( dialog );
 
 		// Check if media is loaded
@@ -661,6 +676,15 @@ std::string KinoCommon::importFile( const char *filename )
 		g_spawn_async_with_pipes( ".", (gchar**) args, NULL, G_SPAWN_DO_NOT_REAP_CHILD,
 			NULL, NULL, &pid, NULL, NULL, NULL, &gerror );
 		
+		// Get the total expected frames from the detected duration
+		int totalFrames = 0;
+		SMIL::MediaClippingTime tempTime( strcmp( args[ 3 ], "pal" ) ? ( 30000.0 / 1001.0 ) : 25.0 );
+		if ( strcmp( duration, "" ) )
+		{
+			tempTime.parseTimeValue( duration );
+			totalFrames = tempTime.getFrames() - 1;
+		}
+
 		// Check for cancellation or child exit
  		while ( ! WIFEXITED( status ) && visible )
 		{
@@ -668,8 +692,39 @@ std::string KinoCommon::importFile( const char *filename )
 			while ( gtk_events_pending() )
 				gtk_main_iteration();
 			nanosleep( &ts, NULL );
-			gtk_progress_bar_pulse( progressBar );
- 			g_object_get( G_OBJECT( dialog ), "visible", &visible, static_cast<gchar*>( NULL ) );
+
+			// Determine size of converted file
+			struct stat fileStatus;
+			if ( stat( importedFile.c_str(), &fileStatus ) == 0 )
+			{
+				int frames = fileStatus.st_size / ( strcmp( args[ 3 ], "pal" ) ? 120000 : 144000 );
+				if ( frames > 0 )
+				{
+					// If we have a valid duration
+					if ( totalFrames > 0 )
+					{
+						double fraction = frames;
+						fraction /= totalFrames;
+						gtk_progress_bar_set_fraction( progressBar, fraction > 1.0 ? 1.0 : fraction );
+					}
+					else
+					{
+						gtk_progress_bar_pulse( progressBar );
+					}
+				
+					// Put the amount converted as a textual time value
+					gtk_progress_bar_set_text( progressBar, tempTime.parseFramesToString( frames, getTimeFormat() ).c_str() );
+				}
+				else
+				{
+					gtk_progress_bar_pulse( progressBar );
+				}
+			}
+			else
+			{
+				gtk_progress_bar_pulse( progressBar );
+			}
+			g_object_get( G_OBJECT( dialog ), "visible", &visible, static_cast<gchar*>( NULL ) );
 			waitpid( pid, &status, WNOHANG );
 		}
 		if ( ! WIFEXITED( status ) )
@@ -2360,8 +2415,9 @@ void KinoCommon::publishPlayList()
 		delete copy;
 		command << "\"" << DATADIR << "/kino/scripts/publish/project.sh\" ";
 		command << "\"" << filename << "\" ";
-		command << "\"" << playlist.GetDocId().c_str() << "\" ";
-		command << "\"" << playlist.GetDocTitle().c_str() << "\" ";
+		command << "\"" << playlist.GetDocId() << "\" ";
+		command << "\"" << playlist.GetDocTitle() << "\" ";
+		command << std::ends;
 		const char * args[ 4 ];
 		args[ 0 ] = "/bin/sh";
 		args[ 1 ] = "-c";
@@ -2375,8 +2431,9 @@ void KinoCommon::publishPlayList()
 		savePlayList();
 		command << "\"" << DATADIR << "/kino/scripts/publish/project.sh\" ";
 		command << "\"" << playlist.GetDocName() << "\" ";
-		command << "\"" << playlist.GetDocId().c_str() << "\" ";
-		command << "\"" << playlist.GetDocTitle().c_str() << "\" ";
+		command << "\"" << playlist.GetDocId() << "\" ";
+		command << "\"" << playlist.GetDocTitle() << "\" ";
+		command << std::ends;
 		const char * args[ 4 ];
 		args[ 0 ] = "/bin/sh";
 		args[ 1 ] = "-c";
@@ -2404,9 +2461,10 @@ void KinoCommon::publishFrame()
 			
 			command << "\"" << DATADIR << "/kino/scripts/publish/frame.sh\" ";
 			command << "\"" << filename << "\" ";
-			command << "\"" << getPlayList()->GetDocId().c_str() << "\" ";
+			command << "\"" << getPlayList()->GetDocId() << "\" ";
 			command << "\"" << getPlayList()->GetSeqAttribute( g_currentFrame, "title" ) << "\" ";
-			command << "\"" << getPlayList()->GetDocTitle().c_str() << "\" ";
+			command << "\"" << getPlayList()->GetDocTitle() << "\" ";
+			command << std::ends;
 			const char * args[ 4 ];
 			args[ 0 ] = "/bin/sh";
 			args[ 1 ] = "-c";
