@@ -29,10 +29,12 @@ extern "C"
 void modal_message ( const char * message, ... );
 char *modal_prompt( const char * message, ... );
 int modal_confirm( const char * affirm_label, const char *close_label, const char *message, ... );
+int modal_confirm_files( const char * affirm_label, const char *close_label, const char* files, const char *message, ... );
 	
 void modal_message_with_parent ( GtkWidget *parent, const char * message, ... );
 char *modal_prompt_with_parent( GtkWidget *parent, const char * message, ... );
 int modal_confirm_with_parent( const char * affirm_label, const char *close_label, GtkWidget *parent, const char *message, ... );
+int modal_confirm_files_with_parent( const char * affirm_label, const char *close_label, GtkWidget *parent, const char* files, const char *message, ... );
 
 #ifdef __cplusplus
 }
