@@ -564,7 +564,7 @@ void PageCapture::videoRewind()
 /** Frame back.
 */
 
-void PageCapture::videoBack()
+void PageCapture::videoBack( int step )
 {
 	if ( avc_enabled )
 	{
@@ -630,13 +630,13 @@ void PageCapture::videoStop()
 /** Forward.
 */
 
-void PageCapture::videoForward()
+void PageCapture::videoForward( int step )
 {
 	if ( avc_enabled )
 	{
 		audioOn = FALSE;
 		gtk_toggle_button_set_active( this->muteButton, !audioOn );
-		common->toggleComponents( VIDEO_FORWARD, avc_enabled );
+		common->toggleComponents( VIDEO_FORWARD, false );
 		if ( avc->Pause( Preferences::getInstance().phyID ) >= 0 )
 			avc->Forward( Preferences::getInstance().phyID );
 	}
