@@ -705,7 +705,7 @@ gboolean PageEditor::processKeyboard( GdkEventKey *event )
   	\param cmd		command to be processed;
 */
 
-gboolean PageEditor::processCommand( char *command )
+gboolean PageEditor::processCommand( const char *command )
 {
 	int	start, end;
 	int count = 1;
