@@ -1,6 +1,7 @@
 /*
 * page_export_mjpeg.cc -- Notebook Firewire/AVI/Still Frame Export Page Object
 * Copyright (C) 2002 Charles Yates <charles.yates@pandora.be>
+* Copyright (C) 2002-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -46,6 +47,16 @@ using std::endl;
 #include "message.h"
 #include "rwpipe.h"
 
+static void
+on_optionmenu_export_mjpeg_format_changed( GtkOptionMenu *widget, gpointer user_data )
+{
+	int i = gtk_option_menu_get_history( widget );
+	GtkWidget* vbox = lookup_widget( GTK_WIDGET( widget ), "vbox_export_mjpeg_dvd_options" );
+	if ( i == 6 )
+		gtk_widget_show( vbox );
+	else
+		gtk_widget_hide( vbox );
+}
 
 class DVDTool : protected RWPipe
 {
@@ -93,7 +104,7 @@ public:
 		bool result;
 
 		// Construct the command
-		command = g_strdup_printf( "%s \"%s\" \"%s\"", m_command.c_str( ),
+		command = g_strdup_printf( "\"%s\" \"%s\" \"%s\"", m_command.c_str( ),
 		         dvdauthorFile.c_str( ), outputDirectory.c_str() );
 
 		result = system( command );
@@ -122,7 +133,8 @@ public:
 ExportMJPEG::ExportMJPEG( PageExport *_exportPage, KinoCommon *_common ) :
 		Export( _exportPage, _common ),
 		prefs( Preferences::getInstance() ),
-		hasYuvDeinterlace( false )
+		hasYuvDeinterlace( false ),
+		isAvailable( true )
 {
 	cerr << "> Creating ExportMJPEG Page" << endl;
 
@@ -166,9 +178,6 @@ ExportMJPEG::ExportMJPEG( PageExport *_exportPage, KinoCommon *_common ) :
 	cleanupCheck
 	= GTK_TOGGLE_BUTTON( lookup_widget( common->getWidget(),
 					"checkbutton_export_mjpeg_cleanup" ) );
-	authorCheck
-	= GTK_TOGGLE_BUTTON( lookup_widget( common->getWidget(),
-	        			"checkbutton_export_mjpeg_author" ) );
 	mjpegAspect
 	= GTK_MENU( gtk_option_menu_get_menu( GTK_OPTION_MENU( lookup_widget( exportPage->getWidget(),
 	                                      "optionmenu_export_aspect_ratio" ) ) ) );
@@ -176,13 +185,17 @@ ExportMJPEG::ExportMJPEG( PageExport *_exportPage, KinoCommon *_common ) :
 	cerr << "> Initializing MJPEG Export Page settings from Preferences" << endl;
 	gtk_option_menu_set_history( GTK_OPTION_MENU( lookup_widget( exportPage->getWidget(),
 								"optionmenu_export_mjpeg_format" ) ), prefs.exportMjpegFormat );
+	GtkWidget* vbox = lookup_widget( GTK_WIDGET( exportPage->getWidget() ), "vbox_export_mjpeg_dvd_options" );
+	if ( prefs.exportMjpegFormat == 6 )
+		gtk_widget_show( vbox );
+	else
+		gtk_widget_hide( vbox );
 	gtk_option_menu_set_history( GTK_OPTION_MENU( lookup_widget( exportPage->getWidget(),
 								"optionmenu_export_mjpeg_deinterlace" ) ), prefs.exportMjpegDeinterlace );
 	gtk_option_menu_set_history( GTK_OPTION_MENU( lookup_widget( exportPage->getWidget(),
 								"optionmenu_export_aspect_ratio" ) ) , prefs.exportMjpegAspect );
 	gtk_toggle_button_set_active( splitCheck, prefs.exportMjpegSceneSplit );
 	gtk_toggle_button_set_active( cleanupCheck, prefs.exportMjpegCleanup );
-	gtk_toggle_button_set_active( authorCheck , prefs.exportMjpegOutputDVDAuthorXML );
 	gtk_entry_set_text( mjpegVideo, prefs.exportMjpegVideoPipe );
 	gtk_entry_set_text( mjpegAudio, prefs.exportMjpegAudioPipe );
 	gtk_entry_set_text( mjpegMultiplexer, prefs.exportMjpegMultiplex );
@@ -208,6 +221,9 @@ ExportMJPEG::ExportMJPEG( PageExport *_exportPage, KinoCommon *_common ) :
 	}
 	gtk_option_menu_set_menu( GTK_OPTION_MENU( lookup_widget( common->getWidget(),
 		"optionmenu_export_mjpeg_deinterlace" ) ), GTK_WIDGET( deinterlaceMenu ) );
+	g_signal_connect ( G_OBJECT( lookup_widget( common->getWidget(),
+		"optionmenu_export_mjpeg_format" ) ), "changed",
+		G_CALLBACK ( on_optionmenu_export_mjpeg_format_changed ), this );
 }
 
 /** Destructor for page.
@@ -218,6 +234,56 @@ ExportMJPEG::~ExportMJPEG()
 	cerr << "> Destroying ExportMJPEG Page" << endl;
 }
 
+void ExportMJPEG::start()
+{
+	// Show DVD options if applicable
+	GtkWidget* vbox = lookup_widget(  common->getWidget(), "vbox_export_mjpeg_dvd_options" );
+	if ( prefs.exportMjpegFormat == 6 )
+		gtk_widget_show( vbox );
+	else
+		gtk_widget_hide( vbox );
+	
+	// Disable UI if mjpegtools not installed.
+	vbox = lookup_widget(  common->getWidget(), "vbox_export_mjpeg" );
+	const char* errorMessage = _( "Failed to execute %s. MPEG export requires a complete installation of mjpegtools." );
+	if ( system( "which mpeg2enc > /dev/null 2> /dev/null" ) )
+	{
+		gtk_widget_set_sensitive( vbox, FALSE );
+		common->setStatusBar( errorMessage, "mpeg2enc" );
+		isAvailable = false;
+		return;
+	}
+	if ( system( "which mp2enc > /dev/null 2> /dev/null" ) )
+	{
+		gtk_widget_set_sensitive( vbox, FALSE );
+		common->setStatusBar( errorMessage, "mp2enc" );
+		isAvailable = false;
+		return;
+	}
+	if ( system( "which mplex > /dev/null 2> /dev/null" ) )
+	{
+		gtk_widget_set_sensitive( vbox, FALSE );
+		common->setStatusBar( errorMessage, "mplex" );
+		isAvailable = false;
+		return;
+	}
+	
+	// Otherwise, enable UI
+	isAvailable = true;
+	gtk_widget_set_sensitive( vbox, TRUE );
+}
+
+/** Define active widgets.
+    If mjpegtools are not installed, do not support any common functions.
+ */
+
+gulong ExportMJPEG::onActivate()
+{
+	if ( isAvailable )
+		return EXPORT_EXPORT | EXPORT_PAUSE | EXPORT_SCENE_LIST;
+	return 0;
+}
+
 /** start exporting MJPEG
  */
 enum export_result
@@ -232,7 +298,6 @@ ExportMJPEG::doExport( PlayList * playlist, int begin, int end, int every,
 	prefs.exportMjpegAspect = g_list_index ( GTK_MENU_SHELL ( mjpegAspect ) ->children, gtk_menu_get_active( mjpegAspect ) );
 	prefs.exportMjpegSceneSplit = gtk_toggle_button_get_active( splitCheck );
 	prefs.exportMjpegCleanup = gtk_toggle_button_get_active( cleanupCheck );
-	prefs.exportMjpegOutputDVDAuthorXML = gtk_toggle_button_get_active( authorCheck );
 	GtkWidget* widget = lookup_widget( common->getWidget(), "optionmenu_dvd_tools" );
 	widget = gtk_option_menu_get_menu( GTK_OPTION_MENU( widget ) );
 	prefs.exportMjpegDvdTool = g_list_index ( GTK_MENU_SHELL( widget )->children, gtk_menu_get_active( GTK_MENU( widget ) ) );
@@ -309,7 +374,8 @@ ExportMJPEG::doExport( PlayList * playlist, int begin, int end, int every,
 	/* only necessary to check this option if we are creating DVD video */
 	if( videoformat == 6 )
 	{
-		author = gtk_toggle_button_get_active( authorCheck );
+		// Tool 0 is always "None"
+		author = prefs.exportMjpegDvdTool > 0;
 	}
 	active_item = gtk_menu_get_active( mjpegAspect );
 	aspectRatio = g_list_index ( GTK_MENU_SHELL ( mjpegAspect ) ->children, active_item );
@@ -351,20 +417,40 @@ ExportMJPEG::doExport( PlayList * playlist, int begin, int end, int every,
 		{
 		case 1:
 			strcpy( scale, "VCD" );
+			frequency = 44100;
+			if ( strstr( audioencode, "ffmpeg" ) )
+				strcpy( audioOptions, "-ar 44100" );
+			else if ( strstr( audioencode, "mp2enc" ) )
+				strcpy( audioOptions, "-r 44100" );
 			break;
 		case 2:
 			strcpy( scale, "VCD" );
 			strcpy( bitrate, "-b 1152" );
+			frequency = 44100;
+			if ( strstr( audioencode, "ffmpeg" ) )
+				strcpy( audioOptions, "-ar 44100" );
+			else if ( strstr( audioencode, "mp2enc" ) )
+				strcpy( audioOptions, "-r 44100" );
 			break;
 		case 3:
 			strcpy( bitrate, "-b 4000" );
 			break;
 		case 4:
 			strcpy( scale, "SVCD" );
+			frequency = 44100;
+			if ( strstr( audioencode, "ffmpeg" ) )
+				strcpy( audioOptions, "-ar 44100" );
+			else if ( strstr( audioencode, "mp2enc" ) )
+				strcpy( audioOptions, "-r 44100" );
 			break;
 		case 5:
 			strcpy( scale, "SVCD" );
 			strcpy( bitrate, "-b 2500" );
+			frequency = 44100;
+			if ( strstr( audioencode, "ffmpeg" ) )
+				strcpy( audioOptions, "-ar 44100" );
+			else if ( strstr( audioencode, "mp2enc" ) )
+				strcpy( audioOptions, "-r 44100" );
 			break;
 		case 6:
 			videoformat = 8;
@@ -410,13 +496,15 @@ ExportMJPEG::doExport( PlayList * playlist, int begin, int end, int every,
 	}
 	if ( strstr( audioencode, "ffmpeg" ) )
 	{
-		audiopipe = g_strdup_printf( "|%s -f wav -i pipe: %s -y \'%s\'.ac3", audioencode, audioOptions, filename );
+		audiopipe = g_strdup_printf( "|%s -f wav -i pipe: %s -y \'%s\'.%s", audioencode, audioOptions,
+			filename, videoformat == 8 ? "ac3" : "mp2" );
 		if ( strcmp( multiplexer, "" ) )
 		{
-			multiplex = g_strdup_printf( "%s -f %d -o \'%s\'%%s.mpeg \'%s\'.mpv \'%s\'.ac3",
-					multiplexer, videoformat, filename, filename, filename );
-			cleanup = g_strdup_printf( "rm -f \'%s\'.mpv \'%s\'.ac3 \'%s\'.wav",
-					filename, filename, filename );
+			multiplex = g_strdup_printf( "%s -f %d -o \'%s\'%%s.mpeg \'%s\'.mpv \'%s\'.%s",
+					multiplexer, videoformat, filename, filename, filename,
+					videoformat == 8 ? "ac3" : "mp2" );
+			cleanup = g_strdup_printf( "rm -f \'%s\'.mpv \'%s\'.%s \'%s\'.wav",
+					filename, filename, videoformat == 8 ? "ac3" : "mp2", filename );
 		}
 	}
 	else if ( strstr( audioencode, "mp2enc" ) )
@@ -591,11 +679,13 @@ ExportMJPEG::doExport( PlayList * playlist, int begin, int end, int every,
      	createAuthorXml( filename, nameList, split, isPAL );
 
 		// Run the selected tool
-		DVDTool *tool = m_dvdTools[ prefs.exportMjpegDvdTool ];
+		// -1 to account for first "None" item
+		DVDTool *tool = m_dvdTools[ prefs.exportMjpegDvdTool - 1 ];
 		if ( tool )
 		{
 			string dvdauthorFile( string( filename ) + "-dvdauthor.xml" );
-			tool->execute( dvdauthorFile, string( filename ) );
+			if ( tool->execute( dvdauthorFile, string( filename ) ) )
+				modal_message( _("The dvdauthor script failed, but the MPEG export succeeded.") );
 			tool->close();
 		}
 	}
@@ -676,12 +766,7 @@ ExportMJPEG::createAuthorXml( const char *filename,
 		chapterList << "0";
 
 		for( sceneIter = sceneStarts.begin(); sceneIter != sceneStarts.end() - 1; ++sceneIter )
-		{
-			std::string tmpSceneStr;
-			tmpSceneStr = this->common->getTime().parseFramesToString( *sceneIter, SMIL::Time::TIME_FORMAT_SMPTE );
-			chapterList << "," << tmpSceneStr.substr( 1, 7 ) << "." << std::setw( 2 ) << std::setfill( '0' ) <<
-				static_cast< int >( atof( tmpSceneStr.substr( 9, 2 ).c_str() ) / ( isPal ? 25 : 29.97 ) * 100 );
-		}
+			chapterList << "," << this->common->getTime().parseFramesToString( *sceneIter, SMIL::Time::TIME_FORMAT_CLOCK );
 
 		/* output the single vob, checking for chapter marks */
 		xml << "\t\t\t\t<vob file=\"" << *iter << "\" chapters=\"" <<
@@ -748,6 +833,11 @@ void ExportMJPEG::activateTool( int index )
 	if ( menu )
 	{
 		menu = GTK_MENU( gtk_menu_new( ) );
+		// Create a "None" menu item
+		GtkWidget *item = gtk_menu_item_new_with_label( _("None") );
+		gtk_widget_show( item );
+		gtk_menu_append( menu, item );
+		// Creat the other menu items
 		for ( unsigned int i = 0; i < m_dvdTools.size( ); i ++ )
 		{
 			GtkWidget *item = gtk_menu_item_new_with_label( m_dvdTools[ i ] ->m_description.c_str( ) );
