@@ -2600,3 +2600,14 @@ bool EditorBackup::Restore( PlayList * playlist )
 	}
 	return result;
 }
+
+string PlayList::GetBaseFileOfFrame( int frameNum ) const
+{
+	char *fname = GetFileNameOfFrame( frameNum );
+	char  bfname[strlen( fname ) + 10];
+	char *ptr = strrchr( fname, '.' );
+	if ( ptr ) *ptr = 0;
+	sprintf( bfname, "%s_%06d", fname, frameNum );
+	free( fname );
+	return bfname;
+}
