@@ -28,6 +28,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <errno.h>
+#include <string.h>
 
 #include <string>
 using std::string;
@@ -184,7 +185,7 @@ Preferences::Preferences() : phyID( -1 )
 	get( displayExtract, "displayExtract", "1" );
 	get( relativeSave, "relativeSave", "true" );
 	get( dvCaptureDevice, sizeof(dvCaptureDevice), "dvCaptureDevice", "/dev/dv1394/0" );
-	get( dv1394Preview, "dv1394Preview", "true" );
+	get( dv1394Preview, "dv1394Preview", "false" );
 	get( dvExportDevice, sizeof(dvExportDevice), "dvExportDevice", "/dev/dv1394/0" );
 	get( avcPollIntervalMs, "avcPollIntervalMs", "200" );
 	get( dvExportPrerollSec, "dvExportPrerollSec", "4" );
